// *****************************************************************
// ****                      - hard plAYer -                    ****
// ****                - hard plAYer's- main module             ****
// ****                  www.tolaemon.com/hplayer               ****
// ****                       Source code                       ****
// ****                       16-10-2008                        ****
// ****                     Jordi Bartolome                     ****
// ****                                                         ****
// **** IMPORTANT:                                              ****
// **** Using parts of this code means accepting all conditions ****
// **** exposed on the web: www.tolaemon.com/hplayer            ****
// *****************************************************************

/* 
&	AND	Bitwise AND
|	OR	Bitwise Inclusive OR
^	XOR	Bitwise Exclusive OR
~	NOT	Unary complement (bit inversion)
<<	SHL	Shift Left
>>	SHR	Shift Right
*/

/* aproximated RAM ocupation sumary:
	GLOBAL VARIABLES:
	LCDG:
		144: 8 page * 18  columns char LCDG_char_buffer[LCDG_PAGES][LCDG_CHAR_COLUMNS]
		  1: uint8_t LCDG_char_buffer_modified
		  1: uint8_t LCDG_Motor_page
		146
	SDC:
		SDMMC_buffer:
		512:	uint8_t sector_buffer[SDMMC_SECTOR_SIZE];
		  4:	uint32_t actual_sector;
		516
	TIMERS:
		  4: uint32_t timer0_calls
		  4
	UART0:
		UART0_buffers:
		  1:	uint8_t recv_write_index; // points to the first writeable postion in the reception buffer
		  1:	uint8_t recv_read_index; // points to the first byte to read in the reception buffer
		 64:	unsigned char recv_buffer[MAX_UART_BUFFER];// is the UART reception buffer
		  1:	uint8_t trans_write_index; // points to the first writeable postion in the transmision buffer
		  1:	uint8_t trans_read_index; // points to the first byte to read in the transmision buffer
		 64:	unsigned char trans_buffer[MAX_UART_BUFFER];// is the UART transmision buffer
		132
	FAT32:
		FAT32_Info:
		  1:	uint8_t state;// 0- Not initialized 1-Initialized 2-Error'. Flag wich is set to 0,1,2... depending on the state of the FAT32 information
		  2:	uint16_t bpb_bytspersec;// FAT32 BPB_BytsPerSec - 2 bytes
		  1:	uint8_t bpb_secperclus;// FAT32 BPB_SecPerClus - 1 byte
		  2:	uint16_t bpb_resvdseccnt;// FAT32 BPB_ResvdSecCnt - 2 bytes , it is the FAT start
		  1:	uint8_t bpb_numfats;// FAT32 BPB_NumFATs - 1 byte
		  4:	uint32_t bpb_fatsz32;// FAT32 BPB_FATSz - 4bytes
		  4:	uint32_t bpb_rootclus;// FAT32 BPB_RootClus - 4 bytes
		  4:	uint32_t bytes_per_cluster;// Precalculated number of bytes per cluster ( bpb_secperclus * bpb_secperclus ) - 4 bytes
		  4:	uint32_t first_data_region_sector;// number of the first sector of data region - 4 bytes
		 23
		FILE_DIR_Ifo:
		  1:	uint8_t type;// 0-It is  a root directory  1-It is  a directory 2-It is a file
		  2:	uint16_t number_of_files_dirs;// if we are in a directory it contains the number of files and directories
		 13:	uint8_t name[MAX_NAME_SHORT_ENTRY+1+1];// file or directory short name, +1+1 is for the '.' and the '\0'
		 13:	uint8_t previous_name[MAX_NAME_SHORT_ENTRY+1+1];// used to store the file or directory short name, +1+1 on which we were before going into the file or directory
		  4:	uint32_t first_fat_entry;// number of the first FAT entry of the directory or file. With this number and the information contained in FAT32_info structure can be read the content of the first FAT entry of the file or directory wich is being read
		  4:	uint32_t previous_first_fat_entry;// used to store the file or directory number of the first FAT entry of the directory or file on which we were before going into the file or directory. With this number and the information contained in FAT32_info structure can be read the content of the first FAT entry of the file or directory wich is being read
		  4:	uint32_t first_sector;// number of the first sector of the first FAT entry of the file or directory
		  4:	uint32_t size;//size in case it is a file
		  4:	uint32_t number_of_read_cluster;// number of cluster in the memory wich is being read
		  4:	uint32_t number_of_read_sector;// number of sector in the memory wich is being read
		  2:	uint16_t number_of_read_byte;// number of byte inside the sector wich is being read
		  4:	uint32_t last_read_byte_offset;// last read byte in the file, it is very usefull to know if is necessary to recalculate the adresses of cluster, sectors or bytes
		 59
	INTERFACE:
		INTERFACE_state_struct:
		  1:	uint8_t state; // controls the state of the Interface Screeen
		  1:	uint8_t lcd_content_modified; // is the flag wich controls if the LCD has to be updated
		  2:	uint16_t file_expl_first;// is the N of the entry of the dir wich apears first on the LCD list
		  2:	uint16_t file_expl_selected;// wich of the N of entries of the list is currently selected
		  2:	uint16_t file_expl_numb_entries;// is the N of entries in the directory
		  1:	uint8_t sw_buttons_state;//stores the state of the sw buttons after last read
		  1:	uint8_t refresh_path_entries;// when this flag is true the LCD buffer is updated with the next NUMB_FILE_EXPL_ENTRIES,
		 10
		short_entry[MAX_NAME_SHORT_ENTRY+1+1]:
		  11:	uint8_t
		long_entry[MAX_LONG_NAME+1+1]:
		  16:	uint8_t
	TOTAL (ARPOX):
	    917 bytes in static vars
		
	LOCAL VARIABLES (APROX):
	Each function or procedure takes an average of 4 bytes of RAM for vars.
	
*/

#include "ayfwre.h"
#include "uart0.h"
#include "lcdg.h"
#include "timers.h"
#include "sdc.h"
#include "fat32.h"
#include "interface.h"

void IOinit(){

	// Declare your local variables here

	// Input/Output Ports initialization
	// Port A initialization
	PORTA=0x00;
	DDRA=0xFF;

	// Port B initialization
	PORTB=0x00;
	DDRB=0x00;

	// Port C initialization
	PORTC=0x00;
	DDRC=0xFF;

	// Port D initialization
	PORTD=0x00;
	DDRD=0x00;

	// Port E initialization
	PORTE=0x00;
	DDRE=0x00;

	// Port F initialization
	PORTF=0x00;
	DDRF=0xE1;// 3 upper bits (7,6,5) and lowest bit (0) are outputs (RESET_LCD,RS_LCD,E_LCD,WRB)  other pins are inputs (SW4,SW3,SW2,SW1)
	ADCSRA=ADCSRA&0x7F;// ADEN is set to 0 to disable Analog to Digital Conversor and use PORTF as I/O port
		
	// Port G initialization
	// Func4=In Func3=In Func2=In Func1=In Func0=In 
	// State4=T State3=T State2=T State1=T State0=T 
	PORTG=0x00;
	DDRG=0x00;

	// External Interrupt(s) initialization
	// INT0: Off
	// INT1: Off
	// INT2: Off
	// INT3: Off
	// INT4: Off
	// INT5: Off
	// INT6: Off
	// INT7: Off
	EICRA=0x00;
	EICRB=0x00;
	EIMSK=0x00;

	// Analog Comparator initialization
	// Analog Comparator: Off
	// Analog Comparator Input Capture by Timer/Counter 1: Off
	// Analog Comparator Output: Off
	ACSR=0x80;
	SFIOR=0x00;

	//!! The application software must write this bit to the desired value twice within four cycles to change its value.
	MCUCSR=MCUCSR|0x80;// FIRST TIME Set the JTD byte to 1 to disable JTAG interface	
	MCUCSR=MCUCSR|0x80;// SECOND TIME Set the JTD byte to 1 to disable JTAG interface

	SREG=0x80;//Global Interruts enabled
		
};//ioinit



int main (void){

	IOinit ();
	TIMERS_Init();
	UART_Init();
	LCDG_Init();
	SDMMC_Init();
	INTERFACE_Init();
	AY3891X_Init();
	
	for (;;){
		UART0_Motor();		
		INTERFACE_Motor();
		AY3891X_Motor();
		LCDG_Motor();		
	};//for

    return (0);
}
