// *****************************************************************
// ****                LCD Driver for LCD display COG           ****
// ****				with Samsung S6B0724 LCD controller         ****
// ****                       Source code                       ****
// ****                       16-10-2008                        ****
// ****                     Jordi Bartolome                     ****
// ****                                                         ****
// **** IMPORTANT:                                              ****
// **** Using parts of this code means accepting all conditions ****
// **** exposed on the atb web: www.tolaemon.com                ****
// *****************************************************************
#ifndef _LCDG_
#define _LCDG_

#include <inttypes.h>
#include <avr/io.h>
#include "iocompat.h"
#include <avr/interrupt.h>
#include <avr/sleep.h>
#include <avr/pgmspace.h>
#include "ayfwre.h"
#include "timers.h"
#include "uart0.h"


// control line defines

#define SET_WRB		PORTF=PORTF|0x01
#define CLR_WRB		PORTF=PORTF&0xFE

#define SET_RS		PORTF=PORTF|0x40
#define CLR_RS		PORTF=PORTF&0xBF

#define SET_EN		PORTF=PORTF|0x80
#define CLR_EN		PORTF=PORTF&0x7F

#define SET_RESET	PORTF=PORTF|0x20
#define CLR_RESET	PORTF=PORTF;//PORTF=PORTF&0xDF


// data line defines

#define LCDG_DATA 	PORTA

#define DISPLAY_OFF 			0xAE //0b10101110
#define DISPLAY_ON 				0xAF //0b10101111
#define INITIAL_DISPLAY_LINE	0x40 //0b01000000
#define SET_PAGE_ADDRESS		0xB0 //0b10110000
#define SET_COL_ADDRESS_MSB		0x10 //0b00010000
#define SET_COL_ADDRESS_LSB		0x00 //0b00000000
#define ADC_SELECT				0xA0 //0b10100000
#define REVERSE_DISPLAY_NORMAL	0xA6 //0b10100110
#define REVERSE_DISPLAY_INVERT	0xA7 //0b10100111 invierte el negro por el blanco y el blanco por el negro
#define ENTIRE_DISPLAY_OFF		0xA4 //0b10100100
#define ENTIRE_DISPLAY_ON		0xA5 //0b10100101
#define LCDG_BIAS_SELECT			0xA2 //0b10100010

#define POWER_CONTROL			0x2F //0b00101000
#define REGULATOR_RES_SELECT	0x23 //0b00100000

#define SET_REF_VOLT_MODE		0x81 //0b10000001
#define SET_REF_VOLT_REGISTER	0x20 //0b00000000

#define LCDG_CONVERTER  	        0x2C //0b00101100
#define LCDG_REGULATOR			0x2E //0b00101110
#define LCDG_FOLLOWER			0x2F //0b00101111

#define RESET					0xE2 //0b11100010
#define SH_SELECT				0xC0 //0b11000000

#define TEST_INSTR1				0x90 //0b10010000
#define TEST_INSTR2				0xF0 //0b11110000

// LCDG characters buffer size
#define LCDG_CHAR_COLUMNS 18
#define LCDG_PAGES 8

#define UNKNOWN_ASCII_SYMBOL '?'

void LCDG_Reset ();
void LCDG_Init();
void LCDG_write_control_byte(uint8_t value);
void LCDG_write_data_byte(uint8_t value);
void LCDG_test(int8_t step);
void LCDG_Motor();
void LCDG_FullFill(uint8_t value);
int8_t LCDG_SetChar(uint8_t col,uint8_t row,uint8_t character);
int8_t LCDG_SetString(uint8_t col,uint8_t row,uint8_t * string);
uint8_t LCD_SetUint(uint8_t col,uint8_t row,uint32_t value,uint8_t num_digits );
int8_t LCDG_SetRow(uint8_t row,uint8_t character);
int8_t LCDG_InvertRow(uint8_t row,uint8_t state);
void LCDG_FillCharBuffer(uint8_t character);
void LCDG_ForceLCDRefresh();
#endif
