// *****************************************************************
// ****			     TIMERS and other timing routines           ****
// ****                     www.tolaemon.com                    ****
// ****                       Source code                       ****
// ****                       16-10-2008                        ****
// ****                     Jordi Bartolome                     ****
// ****                                                         ****
// **** IMPORTANT:                                              ****
// **** Using parts of this code means accepting all conditions ****
// **** exposed on the web: www.tolaemon.com/hplayer            ****
// *****************************************************************

#include "timers.h"



volatile uint32_t timer0_calls=0;



/*	Description:
	Is the ISR of the timer 0.Take a look to TIMERS_Init to know the interval interrupt
	Recives:
	Returns:
	 By value:
	 By reference:
 */
//ISR(SIG_OVERFLOW0){
ISR(SIG_OUTPUT_COMPARE0){
	
	TCNT0=0;
	timer0_calls++;
	
}//ISR(SIG_OVERFLOW0)


/*	Description:
	Returns the value of Timer0. Take a look to TIMERS_Init to know the interval interrupt
	Recives:
	Returns:
		by value the number of interrupts of TIMER0
 */
uint32_t TIMERS_Timer0GetTime(){

	return timer0_calls;

};//TIMERS_Timer0GetTime



/*	Description:
	Returns 1 or -1 if the time recived by  has been reached
	Recives:
	Returns:
		by value: 
			1 if recived time has been reached
			-1 if recived time has not been reached yet
 */
int8_t TIMERS_Timer0ReachedTime(uint32_t time_to_reach){
	int8_t return_value;
		
	if (timer0_calls<time_to_reach){
		return_value=-1;
	}else{
		return_value=1;
	};//if;
	
	return return_value;
	
};//TIMERS_Timer0ReachedTime



/*	Description:
	Implements an active pause until the timer reaches actual_time + ciclos
	Recives:
	Returns:
	 By value:
	 By reference:
 */
void TIMERS_BlockingPause(uint16_t ciclos){
	uint32_t ui32_aux;
	int8_t reached=-1;
	
	ui32_aux=TIMERS_Timer0GetTime();
	while (reached==-1){
		reached=TIMERS_Timer0ReachedTime(ui32_aux+ciclos);
	};//whiles	

};//TIMERS_BlockingPause


/*	Description:
	Executes NumbrerOfPauses pauses of cicles duration each one. It allows to
	execute long pauses, because the standard BlockingPause only allows pauses
	up to a uint16_t range
	Recives:
	Returns:
	 By value:
	 By reference:
 */
void TIMERS_NBlockingPauses(uint8_t number_of_pauses,uint16_t cicles){
	uint32_t ui32_aux;
	int8_t reached=-1;
	int8_t i;
	
	for (i=0;i<number_of_pauses;i++){
		ui32_aux=TIMERS_Timer0GetTime();
		while (reached==-1){
			reached=TIMERS_Timer0ReachedTime(ui32_aux+cicles);
		};//whiles	
	};//for

};//TIMERS_BlockingPause


/*	Description:
	Implements an active pause ( executing nops )
	Recives:
	Returns:
	 By value:
	 By reference:
 */
void TIMERS_NopsBlockingPause(uint32_t ciclos){
	uint32_t i;
	
	for (i=0;i<ciclos;i++){
		NOP;
	};//for
	
};//TIMERS_NopsBlockingPause



/*	Description:
	Initializes all TIMERs
	Recives:
	Returns:
 */
void TIMERS_Init(){

	// Timer/Counter 0 initialization
	// If system clock is 16Mhz, preescaler is at Clck/8, and interrupt ocurs when counter reachs
	// 8 the interrupt will be executed 16E6/1/68= Hz or 235294 times per second, so it will be
	// executed each 1seg/235294 = 0.0425ms
	// TCCR bits in order 7-FOC0 6-WGM00 5-COM01 4-COM00 3-WGM01 2-CS02 1-CS01 0-CS00
	TCCR0=0x09;//FOC0=0 WGM0:1=2 (counter is cleared to zero when the counter matches OCR0 WGM0:1=2 CTC mode), Csk2:0=0 to set preescaler to Clck/1
	ASSR=0x00;
	TCNT0=0x00;// start counting from 0
	OCR0=0x44;// set the Output Compare Register to 68

	// Timer/Counter 1 initialization
	// Clock source: System Clock
	// Clock value: Timer 1 Stopped
	// Mode: Normal top=FFFFh
	// OC1A output: Discon.
	// OC1B output: Discon.
	// OC1C output: Discon.
	// Noise Canceler: Off
	// Input Capture on Falling Edge
	ICR1H=0x00;
	ICR1L=0x00;
	OCR1BH=0x00;
	OCR1BL=0x00;
	OCR1CH=0x00;
	OCR1CL=0x00;

	TCNT1H=0x00;// Start counting from 0
	TCNT1L=0x00;
	TCCR1A=0x00;
	TCCR1B=0x0D;// WGM12=1 CTC mode , CS12=1 CS11=0 CS10=1 clkI/1024
	OCR1AH=0x03;
	OCR1AL=0xFF;// OCR1A to 0x55

	// Timer/Counter 2 initialization
	// Clock source: System Clock
	// Clock value: Timer 2 Stopped
	// Mode: Normal top=FFh
	// OC2 output: Disconnected
	TCCR2=0x00;
	TCNT2=0x00;
	OCR2=0x00;

	// Timer/Counter 3 initialization
	// Clock source: System Clock
	// Clock value: Timer 3 Stopped
	// Mode: Normal top=FFFFh
	// Noise Canceler: Off
	// Input Capture on Falling Edge
	// OC3A output: Discon.
	// OC3B output: Discon.
	// OC3C output: Discon.
	TCCR3A=0x00;
	TCCR3B=0x00;
	TCNT3H=0x00;
	TCNT3L=0x00;
	ICR3H=0x00;
	ICR3L=0x00;
	OCR3AH=0x00;
	OCR3AL=0x00;
	OCR3BH=0x00;
	OCR3BL=0x00;
	OCR3CH=0x00;
	OCR3CL=0x00;

	// Timer(s)/Counter(s) Interrupt(s) initialization
	// OCIE2 TOIE2 TICIE1 OCIE1A OCIE1B TOIE1 OCIE0 TOIE0 
	TIMSK=TIMSK|0x02;// OCIE1A=1 OCIE0=1 Timer 1 Compare Match Interrupt Disabled and Timer 0 Compare Match Interrupt Enabled
	ETIMSK=0x00;

	timer0_calls=0;

};//TIMER_Init
